/**
 * Copyright (c) 2010, Gaudenz Alder, David Benson
 */
/**
 *      Not a Contribution
 *
 *      EXHIBIT A -Mozilla Public License.
 *
 *      "The contents of this file are subject to the Mozilla Public License
 *      Version 1.1 (the "License"); you may not use this file except in
 *      compliance with the License. You may obtain a copy of the License at
 *      https://www.mozilla.org/MPL/
 *
 *      Software distributed under the License is distributed on an "AS IS"
 *      basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See the
 *      License for the specific language governing rights and limitations
 *      under the License.
 *
 *      The Original Code is not modified but introduced data members - shape, edgeStartMarker, edgeEndMarker and
 *		Setters and Getters for these data members.
 *
 *      The Initial Developer of the Original Code is Gaudenz Alder.
 *      Portions created by Siemens are Copyright (C) Siemens 2020. All Rights Reserved.
 *
 *      Contributor(s): Siemens.
 *
 *      Alternatively, the contents of this file may be used under the terms
 *      of the Apache license (the  "Apache License"), in which case the
 *      provisions of Apache License are applicable instead of those
 *      above.  If you wish to allow use of your version of this file only
 *      under the terms of the Apache License and not to allow others to use
 *      your version of this file under the MPL, indicate your decision by
 *      deleting  the provisions above and replace  them with the notice and
 *      other provisions required by the Apache License.  If you do not delete
 *      the provisions above, a recipient may use your version of this file
 *      under either the MPL or the Apache License."
 *
 *      [NOTE: The text of this Exhibit A may differ slightly from the text of
 *      the notices in the Source Code files of the Original Code. You should
 *      use the text of this Exhibit A rather than the text found in the
 *      Original Code Source Code for Your Modifications.]
 *
 */
package com.mxgraph.shape;

import java.awt.Color;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.awt.Graphics2D;

import com.mxgraph.canvas.mxGraphics2DCanvas;
import com.mxgraph.util.mxConstants;
import com.mxgraph.util.mxLine;
import com.mxgraph.util.mxPoint;
import com.mxgraph.util.mxUtils;
import com.mxgraph.view.mxCellState;

public class mxConnectorShape extends mxBasicShape
{

	/**
	 * 
	 */
	public void paintShape(mxGraphics2DCanvas canvas, mxCellState state)
	{
		if (state.getAbsolutePointCount() > 1
				&& configureGraphics(canvas, state, false))
		{
			List<mxPoint> pts = new ArrayList<mxPoint>(
					state.getAbsolutePoints());
			Map<String, Object> style = state.getStyle();

			// Paints the markers and updates the points
			// Switch off any dash pattern for markers
			boolean dashed = mxUtils.isTrue(style, mxConstants.STYLE_DASHED);
			Object dashedValue = style.get(mxConstants.STYLE_DASHED);

			if (dashed)
			{
				style.remove(mxConstants.STYLE_DASHED);
				canvas.getGraphics().setStroke(canvas.createStroke(style));
			}

			Graphics2D prevGraphics = canvas.getGraphics();
			Graphics2D graphicsCopy = (Graphics2D)prevGraphics.create();

			try {
				canvas.setGraphics(graphicsCopy);
				this.translatePoint(pts, 0, this.paintMarker(canvas, state, true));
				this.translatePoint(pts, pts.size() - 1, this.paintMarker(canvas, state, false));
			} finally {
				canvas.setGraphics(prevGraphics);
				graphicsCopy.dispose();
			}


			if (dashed)
			{
				// Replace the dash pattern
				style.put(mxConstants.STYLE_DASHED, dashedValue);
				canvas.getGraphics().setStroke(canvas.createStroke(style));
			}

			paintPolyline(canvas, pts, state.getStyle());
		}
	}

	/**
	 * 
	 */
	protected void paintPolyline(mxGraphics2DCanvas canvas,
			List<mxPoint> points, Map<String, Object> style)
	{
		boolean rounded = isRounded(style)
				&& canvas.getScale() > mxConstants.MIN_SCALE_FOR_ROUNDED_LINES;

		canvas.paintPolyline(points.toArray(new mxPoint[points.size()]),
				rounded);
	}

	/**
	 * 
	 */
	public boolean isRounded(Map<String, Object> style)
	{
		return mxUtils.isTrue(style, mxConstants.STYLE_ROUNDED, false);
	}

	/**
	 * 
	 */
	private void translatePoint(List<mxPoint> points, int index, mxPoint offset)
	{
		if (offset != null)
		{
			mxPoint pt = (mxPoint) points.get(index).clone();
			pt.setX(pt.getX() + offset.getX());
			pt.setY(pt.getY() + offset.getY());
			points.set(index, pt);
		}
	}

	/**
	 * Draws the marker for the given edge.
	 * 
	 * @return the offset of the marker from the end of the line
	 */
	public mxPoint paintMarker(mxGraphics2DCanvas canvas, mxCellState state, boolean source)
	{
		Map<String, Object> style = state.getStyle();
		float strokeWidth = (float) (mxUtils.getFloat(style,
				mxConstants.STYLE_STROKEWIDTH, 1) * canvas.getScale());
		String type = mxUtils.getString(style,
				(source) ? mxConstants.STYLE_STARTARROW
						: mxConstants.STYLE_ENDARROW, "");
		float size = (mxUtils.getFloat(style,
				(source) ? mxConstants.STYLE_STARTSIZE
						: mxConstants.STYLE_ENDSIZE,
				mxConstants.DEFAULT_MARKERSIZE));
		Color color = mxUtils.getColor(style, mxConstants.STYLE_STROKECOLOR);
		canvas.getGraphics().setColor(color);

		double absSize = size * canvas.getScale();

		List<mxPoint> points = state.getAbsolutePoints();
		mxLine markerVector = getMarkerVector(points, source, absSize);
		mxPoint p0 = new mxPoint(markerVector.getX(), markerVector.getY());
		mxPoint pe = markerVector.getEndPoint();

		mxPoint offset = null;

		// Computes the norm and the inverse norm
		double dx = pe.getX() - p0.getX();
		double dy = pe.getY() - p0.getY();

		double dist = Math.max(1, Math.sqrt(dx * dx + dy * dy));
		double unitX = dx / dist;
		double unitY = dy / dist;
		double nx = unitX * absSize;
		double ny = unitY * absSize;

		// Allow for stroke width in the end point used and the 
		// orthogonal vectors describing the direction of the
		// marker
		double strokeX = unitX * strokeWidth;
		double strokeY = unitY * strokeWidth;
		pe = (mxPoint) pe.clone();
		pe.setX(pe.getX() - strokeX / 2.0);
		pe.setY(pe.getY() - strokeY / 2.0);
		
		mxIMarker marker = mxMarkerRegistry.getMarker(type);
		
		if (marker != null)
		{
			offset = marker.paintMarker(canvas, state, type, pe, nx, ny, absSize, source);
			
			if (offset != null)
			{
				offset.setX(offset.getX() - strokeX / 2.0);
				offset.setY(offset.getY() - strokeY / 2.0);
			}
		}
		else
		{
			// Offset for the strokewidth
			nx = dx * strokeWidth / dist;
			ny = dy * strokeWidth / dist;

			offset = new mxPoint(-strokeX / 2.0, -strokeY / 2.0);
		}

		return offset;
	}

	/**
	 * Hook to override creation of the vector that the marker is drawn along
	 * since it may not be the same as the vector between any two control
	 * points
	 * @param points the guide points of the connector
	 * @param source whether the marker is at the source end
	 * @param markerSize the scaled maximum length of the marker
	 * @return a line describing the vector the marker should be drawn along
	 */
	protected mxLine getMarkerVector(List<mxPoint> points, boolean source,
			double markerSize)
	{
		int n = points.size();
		mxPoint p0 = (source) ? points.get(1) : points.get(n - 2);
		mxPoint pe = (source) ? points.get(0) : points.get(n - 1);
		int count = 1;
		
		// Uses next non-overlapping point
		while (count < n - 1 && Math.round(p0.getX() - pe.getX()) == 0 && Math.round(p0.getY() - pe.getY()) == 0)
		{
			p0 = (source) ? points.get(1 + count) : points.get(n - 2 - count);
			count++;
		}
		
		return new mxLine(p0, pe);
	}
	
}
